#include <string.h>
#include <zephyr/kernel.h>
#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/sys/printk.h>

#include <zephyr/drivers/uart.h>

#include <zephyr/logging/log.h>

#include <dk_buttons_and_leds.h>

LOG_MODULE_REGISTER(log_, LOG_LEVEL_DBG);

// 1000 msec = 1 sec
#define SLEEP_TIME_MS   1000

//the size of the receive buffer 
#define RECEIVE_BUFF_SIZE1 52
#define RECEIVE_BUFF_SIZE2 64
//the receiving timeout period 
#define RECEIVE_TIMEOUT 100

int ret;

// Get the device pointer of the UART hardware 
const struct device *uart= DEVICE_DT_GET(DT_NODELABEL(uart1));

// Define the transmission buffer, which is a buffer to hold the data to be sent over UART */
static uint8_t tx_buf[] =   {"Hi, some data was sent.\n\r"};


#define LED0_NODE DT_ALIAS(led0)
#define LED1_NODE DT_ALIAS(led1)

static const struct gpio_dt_spec led0 = GPIO_DT_SPEC_GET(LED0_NODE, gpios);
static const struct gpio_dt_spec led1 = GPIO_DT_SPEC_GET(LED1_NODE, gpios);

// SW0_NODE is the devicetree node identifier for the "sw0" alias 
/*
#define SW0_NODE	DT_ALIAS(sw0) 
static const struct gpio_dt_spec button = GPIO_DT_SPEC_GET(SW0_NODE, gpios);
*/

// Define the receive buffer 
static uint8_t rx_buf1[RECEIVE_BUFF_SIZE1] = {0};
static uint8_t rx_buf2[RECEIVE_BUFF_SIZE2] = {0};
static uint8_t temp[RECEIVE_BUFF_SIZE2] = {0};

int ret;


static void button_handler(uint32_t button_states, uint32_t has_changed)
{
	uint32_t buttons_pressed = has_changed & button_states;

	if (buttons_pressed & DK_BTN1_MSK) {

		ret = uart_tx(uart, tx_buf, sizeof(tx_buf), SYS_FOREVER_MS);
		if (ret) {
			LOG_INF("Error callback button uart tx!");
			return;
		}
	} else if (buttons_pressed & DK_BTN2_MSK) {
		uart_rx_disable(uart);
		ret = uart_rx_enable(uart ,rx_buf1,sizeof rx_buf1,RECEIVE_TIMEOUT);
		if (ret) {
			LOG_INF("BTN2: error buffer 1");
			return ;
		}
		LOG_INF("BTN2: uart buffer 1 successfully changed.");
	} else if (buttons_pressed & DK_BTN3_MSK) {
		uart_rx_disable(uart);
		ret = uart_rx_enable(uart ,rx_buf2,sizeof rx_buf2,RECEIVE_TIMEOUT);
		if (ret) {
			LOG_INF("BTN3: error buffer 2");
			return ;
		}
		LOG_INF("BTN3: uart buffer 2 successfully changed.");
	} else if (buttons_pressed & DK_BTN4_MSK) {
		LOG_INF("BTN4: No function!");
	}
}


#pragma GCC push_options
//#pragma GCC optimize ("O0")
static void uart_cb(const struct device *dev, struct uart_event *evt, void *user_data)
{

	switch (evt->type) {

	case UART_RX_RDY:

		memcpy(temp, &evt->data.rx.buf[evt->data.rx.offset], 25); // copy it to the other buffer
		//LOG_INF("button pressed");
		LOG_INF("evt->data.rx.len:%d",evt->data.rx.len);
		LOG_INF("data:%s",temp);
		for(size_t i = 0;i<evt->data.rx.len;i++)
		{
			//rx_buf1[i] = 0; 
			temp[i] = 0; 
		}

		gpio_pin_toggle_dt(&led1);
		break;

	case UART_RX_DISABLED:
		uart_rx_enable(dev ,rx_buf1,sizeof rx_buf1,RECEIVE_TIMEOUT);
		break;
		
	default:
		break;
	}
}
#pragma GCC pop_options

int main(void)
{
	LOG_INF("Lets start!, board: %s",CONFIG_BOARD);
	
	//Verify that the UART device is ready 
	if (!device_is_ready(uart)){
		
		return 1 ;
	}

	ret = gpio_pin_configure_dt(&led0, GPIO_OUTPUT_ACTIVE);
	if (ret < 0) {
		LOG_INF("led 0 not initialized.\r\n");
		return 1; 
	}

	ret = gpio_pin_configure_dt(&led1, GPIO_OUTPUT_ACTIVE);
	if (ret < 0) {
		LOG_INF("led 1 not initialized.\r\n");
		return 1; 
	}

	/*

		if (!device_is_ready(button.port)) {
		LOG_INF("Button not initialized.\r\n");
		return 1 ;
	}

	ret = gpio_pin_configure_dt(&button, GPIO_INPUT | GPIO_PULL_UP);
	if (ret < 0) {
		return 1;
	}

	//Configure the interrupt on the button's pin
	ret = gpio_pin_interrupt_configure_dt(&button, GPIO_INT_EDGE_TO_ACTIVE );

	//  Initialize the static struct gpio_callback variable   
    gpio_init_callback(&button_cb_data, button_pressed, BIT(button.pin)); 	
	
	// Add the callback function by calling gpio_add_callback()   
	gpio_add_callback(button.port, &button_cb_data);
	*/

	ret = dk_buttons_init(button_handler);
	if (ret) {
		LOG_ERR("dk_buttons_init, error: %d", ret);
	}

	// Register the UART callback function 
	ret = uart_callback_set(uart, uart_cb, NULL);
	if (ret) {
		LOG_INF("uart_callback_set error, %d\r\n", ret);
		return 1;
	}


	//Start receiving by calling uart_rx_enable() and pass it the address of the receive  buffer 
	ret = uart_rx_enable(uart ,rx_buf1,sizeof rx_buf1,RECEIVE_TIMEOUT);
	if (ret) {
		return 1;
	}
	
	while (1) {
		gpio_pin_toggle_dt(&led0);
		k_msleep(SLEEP_TIME_MS);
	}

}